"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const inversify_1 = require("inversify");
const channels_1 = require("../constants/channels");
const symbols_1 = require("../constants/symbols");
const broker_event_factory_1 = require("../helpers/broker-event.factory");
const broker_event_subscriber_1 = require("./event-subscriber/broker-event-subscriber");
let DoveClient = class DoveClient {
    constructor(ipcTransport, middlewareExecutorFactory, executionContextFactory, brokerResponseListenerFactory) {
        this.ipcTransport = ipcTransport;
        this.middlewareExecutorFactory = middlewareExecutorFactory;
        this.executionContextFactory = executionContextFactory;
        this.brokerResponseListenerFactory = brokerResponseListenerFactory;
        this.middleware = [];
    }
    setMiddleware(middlewares) {
        this.middleware = middlewares;
    }
    subscribe(pattern, listener) {
        return new broker_event_subscriber_1.BrokerEventSubscriber(pattern, (event) => {
            listener(event.data, event);
        });
    }
    send(pattern, data) {
        const { middlewareExecutor, executionContext } = this.prepareMiddlewareContext(pattern, data);
        middlewareExecutor.executeWithoutResponse(executionContext, () => {
            this.ipcTransport.send(channels_1.BROKER_EVENT, executionContext.brokerEvent);
        });
    }
    invoke(pattern, data) {
        return __awaiter(this, void 0, void 0, function* () {
            return (yield this.invokeForBrokerEvent(pattern, data)).data;
        });
    }
    invokeForBrokerEvent(pattern, data) {
        return __awaiter(this, void 0, void 0, function* () {
            const { middlewareExecutor, executionContext } = this.prepareMiddlewareContext(pattern, data);
            return yield middlewareExecutor.execute(executionContext, () => __awaiter(this, void 0, void 0, function* () {
                this.ipcTransport.send(channels_1.BROKER_EVENT, executionContext.brokerEvent);
                return yield this.listenForResponse(executionContext.brokerEvent);
            }));
        });
    }
    prepareMiddlewareContext(pattern, data) {
        const brokerEvent = broker_event_factory_1.BrokerEventFactory.createBrokerEvent(pattern, data);
        const middlewareExecutor = this.middlewareExecutorFactory(this.middleware);
        const executionContext = this.executionContextFactory(brokerEvent);
        return { middlewareExecutor, executionContext };
    }
    listenForResponse(brokerEvent) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.brokerResponseListenerFactory(brokerEvent).listen();
        });
    }
};
DoveClient = __decorate([
    inversify_1.injectable(),
    __param(0, inversify_1.inject(symbols_1.Symbols.IpcTransport)),
    __param(1, inversify_1.inject(symbols_1.Symbols.MiddlewareExecutorFactory)),
    __param(2, inversify_1.inject(symbols_1.Symbols.ClientExecutionContextFactory)),
    __param(3, inversify_1.inject(symbols_1.Symbols.BrokerResponseListenerFactory)),
    __metadata("design:paramtypes", [Object, Function, Function, Function])
], DoveClient);
exports.default = DoveClient;
