"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.BrokerResponseListener = void 0;
const request_timeout_error_1 = require("../../errors/request-timeout.error");
const listener_factory_1 = require("../listener-adapter/factory/listener-factory");
class BrokerResponseListener {
    constructor(brokerEvent) {
        this.brokerEvent = brokerEvent;
        this.PROMISE_TIMEOUT = 30;
        this.listenerAdapter = listener_factory_1.ListenerFactory.createListener();
    }
    listen() {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                return yield this.listenWithTimeout();
            }
            catch (err) {
                throw err;
            }
            finally {
                this.cleanUp();
            }
        });
    }
    cleanUp() {
        this.listenerAdapter.removeListener();
        clearTimeout(this.timeout);
    }
    listenWithTimeout() {
        return new Promise((resolve, reject) => {
            this.listenerAdapter.listen(this.brokerEvent.pattern, this.createResponseListener(resolve));
            this.setResponseTimeout(reject);
        });
    }
    createResponseListener(resolve) {
        return (response) => {
            if (this.isExpectedResponse(response)) {
                resolve(response);
            }
        };
    }
    isExpectedResponse(response) {
        return response.eventId === this.brokerEvent.eventId && response.type === 'RESPONSE';
    }
    setResponseTimeout(reject) {
        this.timeout = setTimeout(() => reject(new request_timeout_error_1.RequestTimeoutError()), this.PROMISE_TIMEOUT * 1000);
    }
}
exports.BrokerResponseListener = BrokerResponseListener;
