"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.IpcTransportComposer = void 0;
const broker_main_adapter_1 = require("../../adapters/broker/broker-main.adapter");
const broker_process_adapter_1 = require("../../adapters/broker/broker-process.adapter");
const broker_renderer_adapter_1 = require("../../adapters/broker/broker-renderer.adapter");
const main_transport_adapter_1 = require("../../adapters/client/main-transport.adapter");
const process_transport_adapter_1 = require("../../adapters/client/process-transport.adapter");
const renderer_transport_adapter_1 = require("../../adapters/client/renderer-transport.adapter");
const dove_mode_enum_1 = require("../../constants/dove-mode.enum");
const symbols_1 = require("../../constants/symbols");
const ipc_process_1 = __importDefault(require("../../process/ipc-process"));
const container_configurable_composer_1 = require("../abstract/container-configurable-composer");
class IpcTransportComposer extends container_configurable_composer_1.ContainerConfiguarableComposer {
    compose() {
        switch (this.config.mode) {
            case dove_mode_enum_1.DoveMode.PROCESS:
                this.container.bind(symbols_1.Symbols.IpcTransport).toConstantValue(new process_transport_adapter_1.ProcessTransportAdapter(new ipc_process_1.default()));
                break;
            case dove_mode_enum_1.DoveMode.RENDERER:
                this.container.bind(symbols_1.Symbols.IpcTransport).to(renderer_transport_adapter_1.RendererTransportAdapter).inSingletonScope();
                break;
            case dove_mode_enum_1.DoveMode.BROKER:
                this.bindBrokerIpcTransport();
                this.container.bind(symbols_1.Symbols.IpcTransport).to(main_transport_adapter_1.MainTransportAdapter).inSingletonScope();
                break;
        }
    }
    bindBrokerIpcTransport() {
        if (this.config.mode === dove_mode_enum_1.DoveMode.BROKER) {
            this.bindProcessAdapters(this.config.options.processes);
            this.bindRendererAdapters(this.config.options.browserWindows);
            this.container.bind(symbols_1.Symbols.BrokerIpcTransport).to(broker_main_adapter_1.BrokerMainAdapter);
        }
    }
    bindProcessAdapters(processes) {
        for (const process of processes) {
            const ipcProcess = new ipc_process_1.default(process);
            this.container.bind(symbols_1.Symbols.BrokerIpcTransport).toConstantValue(new broker_process_adapter_1.BrokerProcessAdapter(ipcProcess));
        }
    }
    bindRendererAdapters(browserWindows) {
        for (const browserWindow of browserWindows) {
            this.container.bind(symbols_1.Symbols.BrokerIpcTransport).toConstantValue(new broker_renderer_adapter_1.BrokerRendererAdapter(browserWindow));
        }
    }
}
exports.IpcTransportComposer = IpcTransportComposer;
