"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ContainerComposition = void 0;
const controllers_metadata_factory_composer_1 = require("./composers/factory/controllers-metadata-factory.composer");
const dove_client_composer_1 = require("./composers/dove-client.composer");
const ipc_transport_composer_1 = require("./composers/ipc-transport.composer");
const metadata_readers_composer_1 = require("./composers/metadata-readers.composer");
const middleware_composer_1 = require("./composers/middleware.composer");
const config_composer_1 = require("./composers/config.composer");
const mode_composer_1 = require("./composers/mode.composer");
const execution_context_factory_composer_1 = require("./composers/factory/execution-context-factory.composer");
const request_executor_factory_composer_1 = require("./composers/factory/request-executor-factory.composer");
const middleware_factory_composer_1 = require("./composers/factory/middleware-factory.composer");
const middleware_executor_factory_composer_1 = require("./composers/factory/middleware-executor-factory.composer");
const broker_response_listener_factory_composer_1 = require("./composers/factory/broker-response-listener-factory.composer");
class ContainerComposition {
    constructor(container, config) {
        this.container = container;
        this.config = config;
    }
    composeDependencies() {
        this.composeConfigDependency();
        this.composeDependenciesInOrder();
    }
    composeConfigDependency() {
        new config_composer_1.ConfigComposer(this.container, this.config).compose();
    }
    composeDependenciesInOrder() {
        for (const ComposerClass of ContainerComposition.composersOrder) {
            const composerInstance = new ComposerClass(this.container);
            composerInstance.compose();
        }
    }
}
exports.ContainerComposition = ContainerComposition;
ContainerComposition.composersOrder = [
    ipc_transport_composer_1.IpcTransportComposer,
    dove_client_composer_1.DoveClientComposer,
    metadata_readers_composer_1.MetadataReadersComposer,
    middleware_factory_composer_1.MiddlewareFactoryComposer,
    middleware_executor_factory_composer_1.MiddlewareExecutorFactoryComposer,
    execution_context_factory_composer_1.ExecutionContextFactoryComposer,
    request_executor_factory_composer_1.RequestExecutorFactoryComposer,
    broker_response_listener_factory_composer_1.BrokerResponseListenerFactoryComposer,
    controllers_metadata_factory_composer_1.ControllersMetadataFactoryComposer,
    middleware_composer_1.MiddlewareComposer,
    mode_composer_1.ModeComposer,
];
