"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const inversify_1 = require("inversify");
const dove_client_1 = __importDefault(require("./client/dove.client"));
const symbols_1 = require("./constants/symbols");
const container_composition_1 = require("./container/container-composition");
class Dove {
    constructor(config) {
        this.setContainer(config);
        this.composeDependencies(config);
    }
    setContainer(config) {
        if (config.container) {
            this.container = config.container;
        }
        else {
            this.container = new inversify_1.Container({ autoBindInjectable: true });
        }
    }
    composeDependencies(config) {
        new container_composition_1.ContainerComposition(this.container, config).composeDependencies();
    }
    start() {
        const moduleMode = this.container.get(symbols_1.Symbols.ModuleMode);
        moduleMode.start();
    }
    setMiddleware(middleware) {
        this.container.rebind(symbols_1.Symbols.GlobalMiddleware).toConstantValue(middleware);
    }
    setControllers(controllers) {
        const moduleConfig = this.container.get(symbols_1.Symbols.IpcModuleConfig);
        moduleConfig.controllers = [...controllers];
    }
    getDoveClient() {
        return this.container.get(dove_client_1.default);
    }
}
exports.default = Dove;
