"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ControllerHandlersMetadataReader = void 0;
require("reflect-metadata");
const inversify_1 = require("inversify");
const decorators_1 = require("../constants/decorators");
const abstract_metadata_reader_1 = require("./abstract-metadata.reader");
let ControllerHandlersMetadataReader = class ControllerHandlersMetadataReader extends abstract_metadata_reader_1.AbstractMetadataReader {
    read(controller) {
        const messageHandlers = {};
        for (const handlerName of this.getControllerProperties(controller)) {
            const messageHandler = controller[handlerName];
            if (this.isReadable(messageHandler, handlerName)) {
                const pattern = this.getHandlerPattern(controller, handlerName);
                if (this.isMessageHandler(pattern)) {
                    const paramsMetadata = this.getHandlerParamsMetadata(controller, handlerName);
                    messageHandlers[pattern] = {
                        controller: controller,
                        handler: messageHandler,
                        paramsMetadata: paramsMetadata ? paramsMetadata : [],
                        middleware: this.getHandlerMiddleware(controller, handlerName),
                    };
                }
            }
        }
        return messageHandlers;
    }
    getControllerProperties(controller) {
        return Object.getOwnPropertyNames(Object.getPrototypeOf(controller));
    }
    isReadable(handler, handlerName) {
        return this.isFunction(handler) && handlerName !== 'constructor';
    }
    getHandlerPattern(controller, handlerName) {
        return Reflect.getMetadata(decorators_1.PATTERN_METADATA, controller[handlerName]);
    }
    getHandlerParamsMetadata(controller, handlerName) {
        return Reflect.getMetadata(decorators_1.HANDLER_ARGS_METADATA, controller[handlerName]);
    }
    getHandlerMiddleware(controller, handlerName) {
        const controllerMiddleware = Reflect.getMetadata(decorators_1.MIDDLEWARE_METADATA, controller);
        const handlerMiddleware = Reflect.getMetadata(decorators_1.MIDDLEWARE_METADATA, controller[handlerName]);
        let middleware = [];
        if (controllerMiddleware) {
            middleware = [...controllerMiddleware];
        }
        if (handlerMiddleware) {
            middleware = [...middleware, ...handlerMiddleware];
        }
        return middleware;
    }
    isMessageHandler(pattern) {
        return pattern !== undefined;
    }
};
ControllerHandlersMetadataReader = __decorate([
    inversify_1.injectable()
], ControllerHandlersMetadataReader);
exports.ControllerHandlersMetadataReader = ControllerHandlersMetadataReader;
