"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.IpcProcessMessageListener = void 0;
const ipc_process_message_interface_1 = require("./ipc-process-message.interface");
class IpcProcessMessageListener {
    constructor(process, channels) {
        this.process = process;
        this.channels = channels;
    }
    start() {
        this.process.on('message', (message) => {
            if (ipc_process_message_interface_1.isIpcProcessMessage(message)) {
                this.forwardToChannel(message);
            }
        });
    }
    forwardToChannel(message) {
        return __awaiter(this, void 0, void 0, function* () {
            const channelListener = this.channels.getChannelListenerByName(message.channelName);
            if (channelListener) {
                yield this.callListenerAndRespond(channelListener, message);
            }
        });
    }
    callListenerAndRespond(channelListener, message) {
        return __awaiter(this, void 0, void 0, function* () {
            const response = yield Promise.resolve(channelListener(message.payload));
            this.respond(message.messageId, response);
        });
    }
    respond(messageId, response) {
        // To avoid infinite loop in main process, we skip channelName property, when sending a response.
        const responseMessage = {
            messageId: messageId,
            payload: response,
        };
        this.process.send(responseMessage);
    }
}
exports.IpcProcessMessageListener = IpcProcessMessageListener;
