"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const uuid_1 = require("uuid");
const ipc_process_channels_1 = require("./ipc-process-channels");
const ipc_process_message_listener_1 = require("./ipc-process-message.listener");
class IpcProcess {
    constructor(childProcess) {
        this.process = childProcess ? childProcess : process;
        this.channels = new ipc_process_channels_1.IpcProcessChannels();
        this.messageListener = new ipc_process_message_listener_1.IpcProcessMessageListener(this.process, this.channels);
        this.messageListener.start();
    }
    send(channelName, payload) {
        this.process.send({ messageId: uuid_1.v4(), channelName: channelName, payload: payload }, (error) => {
            if (error) {
                console.error(error);
            }
        });
    }
    on(channel, listener) {
        if (this.channels.isChannelFree(channel)) {
            this.channels.addChannel(channel, listener);
        }
        else {
            throw new Error(`Listener for channel "${channel}" already exists.`);
        }
    }
}
exports.default = IpcProcess;
